/*
 *
 *
 * Copyright (C) 2008 Benjamin Scott   <benscott@nwlink.com>
 *
 * This file is part of the Kvpm project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License,  version 3, as
 * published by the Free Software Foundation.
 *
 * See the file "COPYING" for the exact licensing terms.
 */

#ifndef REMOVEMIRRORLEG_H
#define REMOVEMIRRORLEG_H


class LogVol;

bool remove_mirror_leg(LogVol *mirrorLeg);


#endif
